<?php
namespace OCA\Dematpayslip\Settings;

use OCP\AppFramework\Http\TemplateResponse;
use OCP\IConfig;
use OCP\IDateTimeFormatter;
use OCP\IDBConnection;
use OCP\IL10N;
use OCP\Share\IManager as IShareManager;
use OCP\IUserManager;
use OCP\Files\IRootFolder;
use OCP\Settings\ISettings;

use OCP\Files\Folder;

use Exception;
use OCA\Dematpayslip\Db\DematpayslipUserMapper;

//use OCA\Pastell\PastellAPI;
//use OCA\Pastell\Db\PastellConfig;
//use OCA\Pastell\Db\PastellConfigMapper;
//use OCA\Pastell\Db\PastellConfigFlow;
use OCA\Pastell\Db\PastellConfigFlowMapper;

class AdminSettings implements ISettings {

    /**
     * @var string App name
     */
    private $_appName;

    /** @var IConfig */
    private $_config;

    /**
     * @var IDBConnection database connection
     */
    private $_dbConnection;

    /** @var IL10N */
    private $l;

    /** @var IDateTimeFormatter */
    private $_dateTimeFormatter;

    /** @var IRootFolder */
    private $_rootFolder;

    /** @var IShareManager */
    private $_shareManager;

    /** @var IUserManager */
    private $_userManager;

    /** @var string User Id */
    private $_userId;

    /** @var DematpayslipUserMapper */
    private $_dematpayslipUserMapper;

    // dematpayslip config
    private $_dematpayslipConfigList = array();

    /**
     * Pastell app exists
     *
     * @var bool
     */
    private $_pastellAppExists = false;

    /** @var array Message List */
    private $_msgList = array(
        'error'   => array(),
        'success' => array(),
        'warning' => array(),
    );

    /**
     * Assign params for template
     *
     * @return  array   Params for template
     */
    private function _assignParams() {
        // demat config
        $this->_dematpayslipConfigList['payslip_dir']          = $this->_config->getAppValue($this->_appName, 'payslip_dir', 'Bulletins'); // Bulletins
        $this->_dematpayslipConfigList['user_id']              = $this->_config->getAppValue($this->_appName, 'user_id', 'demat'); // demat user
        $this->_dematpayslipConfigList['app_data_dir']         = $this->_config->getAppValue($this->_appName, 'app_data_dir', 'dematpayslip'); // dematpayslip directory of user demat (upload, check, report, etc)
        $this->_dematpayslipConfigList['pastell_use']          = $this->_config->getAppValue($this->_appName, 'pastell_use', ''); // uses pastell
        $this->_dematpayslipConfigList['pastell_config_f1_id'] = $this->_config->getAppValue($this->_appName, 'pastell_config_f1_id'); // archiving for 5 years
        $this->_dematpayslipConfigList['pastell_config_f2_id'] = $this->_config->getAppValue($this->_appName, 'pastell_config_f2_id'); // archiving for 50 years

        // pastell config mapper
        $this->_pastellConfigFlowMapper = null;
        $this->_pastellConfigFlowList = array();
        if ($this->_pastellAppExists === true) {
            //$pastellUse = intval($this->_dematpayslipConfigList['pastell_use']);
            //if ($pastellUse === 1) {
                $this->_pastellConfigFlowMapper = new PastellConfigFlowMapper($this->_dbConnection);
                $this->_pastellConfigFlowList = $this->_pastellConfigFlowMapper->findAll();
            //}
        }

        return array(
            'dematpayslipConfigList' => $this->_dematpayslipConfigList,
            'pastellConfigFlowList'  => $this->_pastellConfigFlowList,
            'msgList'                => $this->_msgList
        );
    }

    /**
     * Get payslip share list
     *
     * @return  array|\OCP\Share\IShare[][]
     * @throws  \OCP\Files\NotFoundException
     */
    private function _payslipShareList() {
        $payslipShareList = array();

        $payslipDir    = $this->_config->getAppValue($this->_appName, 'payslip_dir');
        $payslipUserId = $this->_config->getAppValue($this->_appName, 'user_id');

        $userFolder   = null;
        try {
            $userFolder = $this->_rootFolder->getUserFolder($payslipUserId);
        } catch (Exception $e) {
            $this->_msgList['error'][] = $this->l->t('Utilisateur non trouvé') . ' : ' . $payslipUserId . "<br />" . $e->getMessage();
        }

        if (empty($this->_msgList['error'])) {
            $payslipShareNode = $userFolder->get($payslipDir);
            if ($payslipShareNode instanceof Folder) {
                $payslipShareList = $this->_shareManager->getSharesInFolder($payslipUserId, $payslipShareNode);
            }
        }

        return $payslipShareList;
    }

    /**
     * Reshare all payslips
     *
     * @throws  \OCP\Files\InvalidPathException
     * @throws  \OCP\Files\NotFoundException
     */
    private function _payslipsReshare() {
        $payslipDir    = $this->_config->getAppValue($this->_appName, 'payslip_dir');
        $payslipUserId = $this->_config->getAppValue($this->_appName, 'user_id');

        $userFolder = null;
        try {
            $userFolder = $this->_rootFolder->getUserFolder($payslipUserId);
        } catch (Exception $e) {
            $this->_msgList['error'][] = $this->l->t('Répertoire utilisateur non trouvé') . ' : ' . $payslipUserId . "<br />" . $e->getMessage();
        }

        if (empty($this->_msgList['error'])) {
            // payslip already shared
            $payslipShareList = $this->_payslipShareList();
            $payslipMatriculeShareNodeIdList = array();
            if (count($payslipShareList) > 0) {
                foreach ($payslipShareList as $payslipShareNodeId => $payslipShareArr) {
                    $payslipMatriculeShareNodeIdList[] = $payslipShareNodeId;
                }
            }

            $payslipFolder = $userFolder->get($payslipDir);
            if ($payslipFolder instanceof Folder) {
                $payslipMatriculeNodeList = $payslipFolder->getDirectoryListing();
                foreach ($payslipMatriculeNodeList as $payslipMatriculeNode) {
                    if ($payslipMatriculeNode instanceof Folder) {
                        $siret = '';
                        $matricule = '';
                        $siretMatricule = $payslipMatriculeNode->getName();
                        $siretMatriculeArr = explode('-', $siretMatricule);
                        if (count($siretMatriculeArr) >= 2) {
                            $siret = $siretMatriculeArr[0];
                            $matricule = $siretMatriculeArr[1];
                        }

                        $dematpayslipUserList = $this->_dematpayslipUserMapper->findAllBySiretAndMatricule($siret, $matricule);
                        if (count($dematpayslipUserList) === 1) {
                            $dematpayslipUser = $dematpayslipUserList[0];

                            // s'il l'utilisateur n'a pas desactive la dematerialisation de son bulletin de paie
                            //if ($dematpayslipUser->getDisabled() != 1) {
                            // verifier que l'utilisateur existe toujours
                            $dematpayslipUserExists = $this->_userManager->userExists($dematpayslipUser->getUserId());
                            if ($dematpayslipUserExists === true) {
                                // verifier si le dossier n'est pas deja partage
                                if (!in_array($payslipMatriculeNode->getId(), $payslipMatriculeShareNodeIdList)) {
                                    // reshare
                                    $payslipMatriculeShare = $this->_shareManager->newShare();
                                    $payslipMatriculeShare->setNode($payslipMatriculeNode);
                                    $payslipMatriculeShare->setShareType(\OC\Share\Share::SHARE_TYPE_USER);
                                    $payslipMatriculeShare->setSharedWith($dematpayslipUser->getUserId());
                                    $payslipMatriculeShare->setPermissions(\OCP\Constants::PERMISSION_READ);
                                    $payslipMatriculeShare->setSharedBy($payslipUserId);
                                    $payslipMatriculeShare = $this->_shareManager->createShare($payslipMatriculeShare);
                                }
                            }
                            //}
                        } else {
                            // error
                            $this->_msgList['error'][] = $this->l->t('Impossible de re-partager le dossier [matricule="' . $matricule . '"] : aucun utilisateur n\'est associé.');
                            break;
                        }
                    }
                }
            }
        }
    }

    /**
     * Unshare all payslips
     *
     * @throws  \OCP\Files\NotFoundException
     * @throws  \OCP\Share\Exceptions\ShareNotFound
     */
    private function _payslipsUnshare() {
        $payslipShareList = self::_payslipShareList();

        if (count($payslipShareList) > 0) {
            foreach ($payslipShareList as $payslipShareArr) {
                if (is_array($payslipShareArr) && count($payslipShareArr)>=1) {
                    $this->_shareManager->deleteShare($payslipShareArr[0]);
                }
            }
        }
    }

    /**
     * AdminSettings constructor.
     *
     * @param   string                  $AppName                    App name
     * @param   IConfig                 $config                     Config
     * @param   IL10N                   $l                          Lang
     * @param   IDateTimeFormatter      $dateTimeFormatter          Date formatter
     * @param   IDBConnection           $dbConnection               DB connection
     * @param   IRootFolder             $rootFolder                 Root folder
     * @param   IShareManager           $shareManager               Share manager
     * @param   IUserManager            $userManager                User manager
     * @param   DematpayslipUserMapper  $dematpayslipUserMapper     Dematpayslip user mapper
     * @param   string                  $UserId                     User Id
     */
    public function __construct($AppName,
                                IConfig $config,
                                IL10N $l,
                                IDateTimeFormatter $dateTimeFormatter,
                                IDBConnection $dbConnection,
                                IRootFolder $rootFolder,
                                IShareManager $shareManager,
                                IUserManager $userManager,
                                DematpayslipUserMapper $dematpayslipUserMapper,
                                $UserId
    ) {
        $this->_appName = $AppName;
        $this->_config = $config;
        $this->l = $l;
        $this->_dateTimeFormatter = $dateTimeFormatter;
        $this->_dbConnection = $dbConnection;
        $this->_rootFolder = $rootFolder;
        $this->_shareManager = $shareManager;
        $this->_userManager = $userManager;
        $this->_dematpayslipUserMapper = $dematpayslipUserMapper;
        $this->_userId = $UserId;

        // check if Pastell app exists
        $pastellAppDir = __DIR__ . '/../../../pastell';
        if (is_dir($pastellAppDir)) $this->_pastellAppExists = true;
    }

    /**
     * Get form
     *
     * @return  TemplateResponse
     *
     * @throws  \OCP\Files\InvalidPathException
     * @throws  \OCP\Files\NotFoundException
     * @throws  \OCP\Share\Exceptions\ShareNotFound
     */
    public function getForm() {
        $action = filter_input(INPUT_GET, 'action', FILTER_SANITIZE_STRING);

        if ($action === 'payslips_config') {
            $dematpayslipDir = filter_input(INPUT_GET, 'dematpayslip_payslip_dir', FILTER_SANITIZE_STRING);
            $dematpayslipUserUserId = filter_input(INPUT_GET, 'dematpayslip_user_user_id', FILTER_SANITIZE_STRING);
            $dematpayslipAppDataDir = filter_input(INPUT_GET, 'dematpayslip_app_data_dir', FILTER_SANITIZE_STRING);
            $this->_config->setAppValue($this->_appName, 'payslip_dir', $dematpayslipDir);
            $this->_config->setAppValue($this->_appName, 'user_id', $dematpayslipUserUserId);
            $this->_config->setAppValue($this->_appName, 'app_data_dir', $dematpayslipAppDataDir);
        } elseif ($action === 'payslips_unshare') {
            $this->_payslipsUnshare();
        } elseif ($action === 'payslips_reshare') {
            $this->_payslipsReshare();
        } elseif ($action === 'pastell_config') {
            $pastellUse           = filter_input(INPUT_GET, 'pastell_use', FILTER_SANITIZE_NUMBER_INT);
            $pastellConfigFlow1Id = filter_input(INPUT_GET, 'pastell_config_f1_id', FILTER_SANITIZE_NUMBER_INT);
            $pastellConfigFlow2Id = filter_input(INPUT_GET, 'pastell_config_f2_id', FILTER_SANITIZE_NUMBER_INT);

            $configPastellUse = $this->_config->getAppValue($this->_appName, 'pastell_use', '');

            $this->_config->setAppValue($this->_appName, 'pastell_use', intval($pastellUse));

            if ($pastellUse == 1 && $configPastellUse == 1) {
                // check parameters
                if (!($pastellConfigFlow1Id > 0)) {
                    $this->_msgList['error'][] = $this->l->t('Le champ "Flux pour archivage sur 5 ans" est manquant.');
                }
                if (!($pastellConfigFlow2Id > 0)) {
                    $this->_msgList['error'][] = $this->l->t('Le champ "Flux pour archivage sur 50 ans" est manquant.');
                }
            }

            if (count($this->_msgList['error']) <= 0) {
                $pastellConfigF1Id = $this->_config->getAppValue($this->_appName, 'pastell_config_f1_id');
                // FIX -- Pastell config flow is NULL
                if (empty($pastellConfigF1Id)) {
                    $this->_config->deleteAppValue($this->_appName,'pastell_config_f1_id');
                }
                $pastellConfigF2Id = $this->_config->getAppValue($this->_appName, 'pastell_config_f2_id');
                // FIX -- Pastell config flow is NULL
                if (empty($pastellConfigF2Id)) {
                    $this->_config->deleteAppValue($this->_appName,'pastell_config_f2_id');
                }
                $this->_config->setAppValue($this->_appName, 'pastell_config_f1_id', $pastellConfigFlow1Id);
                $this->_config->setAppValue($this->_appName, 'pastell_config_f2_id', $pastellConfigFlow2Id);
            }
        }

        if (!empty($action)) {
            if (count($this->_msgList['error']) <= 0) {
                $this->_msgList['success'][] = $this->l->t('Opération réussie.');
            }
        }

        return new TemplateResponse('dematpayslip', 'settings/admin', $this->_assignParams());
    }

    /**
     * @return string the section ID, e.g. 'sharing'
     */
    public function getSection() {
        return 'dematpayslip';
    }

    /**
     * @return int whether the form should be rather on the top or bottom of
     * the admin section. The forms are arranged in ascending order of the
     * priority values. It is required to return a value between 0 and 100.
     */
    public function getPriority() {
        return 50;
    }
}